@@ -17,7 +17,8 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use std::collections::HashMap;
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
+use std::sync::Arc;
 
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::shard_created::{
@@ -27,16 +28,19 @@ use nucliadb_core::protos::{OpStatus, Resource, ResourceId, VectorSetId, VectorS
 use nucliadb_core::tracing::{self, *};
 use nucliadb_core::{thread, IndexFiles};
 use nucliadb_procs::measure;
-use tokio::sync::Mutex;
+use nucliadb_vectors::VectorErr;
+use tokio::sync::{Mutex, MutexGuard};
 
 use crate::disk_structure::*;
-use crate::shards::metadata::{ShardMetadata, Similarity};
+use crate::shards::metadata::ShardMetadata;
 use crate::shards::versions::Versions;
 use crate::telemetry::run_with_telemetry;
 
+pub struct BlockingToken<'a>(MutexGuard<'a, ()>);
+
 #[derive(Debug)]
 pub struct ShardWriter {
-    pub metadata: ShardMetadata,
+    pub metadata: Arc<ShardMetadata>,
     pub id: String,
     pub path: PathBuf,
     text_writer: TextsWriterPointer,
@@ -54,17 +58,15 @@ pub struct ShardWriter {
 impl ShardWriter {
     #[tracing::instrument(skip_all)]
     fn initialize(
-        id: String,
-        path: &Path,
-        metadata: ShardMetadata,
+        metadata: Arc<ShardMetadata>,
         tsc: TextConfig,
         psc: ParagraphConfig,
         vsc: VectorConfig,
         rsc: RelationConfig,
     ) -> NodeResult<ShardWriter> {
         let versions = Versions::load_or_create(
-            &path.join(VERSION_FILE),
-            metadata.channel.unwrap_or_default(),
+            &metadata.shard_path().join(VERSION_FILE),
+            metadata.channel(),
         )?;
         let text_task = || Some(versions.get_texts_writer(&tsc));
         let paragraph_task = || Some(versions.get_paragraphs_writer(&psc));
@@ -98,9 +100,9 @@ impl ShardWriter {
         let relations = relation_result.transpose()?;
 
         Ok(ShardWriter {
-            id,
+            id: metadata.id(),
+            path: metadata.shard_path(),
             metadata,
-            path: path.to_path_buf(),
             text_writer: fields.unwrap(),
             paragraph_writer: paragraphs.unwrap(),
             vector_writer: vectors.unwrap(),
@@ -147,35 +149,40 @@ impl ShardWriter {
             i => panic!("Unknown relation version {i}"),
         }
     }
-    pub fn clean_and_create(id: String, path: &Path) -> NodeResult<ShardWriter> {
-        let metadata = ShardMetadata::open(&path.join(METADATA_FILE))?;
-        std::fs::remove_dir_all(path)?;
-        std::fs::create_dir(path)?;
+    pub fn clean_and_create(metadata: Arc<ShardMetadata>) -> NodeResult<ShardWriter> {
+        let path = metadata.shard_path();
+        std::fs::remove_dir_all(path.clone())?;
+        std::fs::create_dir(path.clone())?;
         let tsc = TextConfig {
-            path: path.join(TEXTS_DIR),
+            path: path.clone().join(TEXTS_DIR),
         };
 
         let psc = ParagraphConfig {
-            path: path.join(PARAGRAPHS_DIR),
+            path: path.clone().join(PARAGRAPHS_DIR),
         };
 
-        let channel = metadata.channel.unwrap_or_default();
+        let channel = metadata.channel();
 
         let vsc = VectorConfig {
             similarity: Some(metadata.similarity()),
-            path: path.join(VECTORS_DIR),
-            vectorset: path.join(VECTORSET_DIR),
+            path: path.clone().join(VECTORS_DIR),
+            vectorset: path.clone().join(VECTORSET_DIR),
             channel,
         };
         let rsc = RelationConfig {
-            path: path.join(RELATIONS_DIR),
+            path: path.clone().join(RELATIONS_DIR),
             channel,
         };
-        ShardWriter::initialize(id, path, metadata, tsc, psc, vsc, rsc)
+        let shard_metadata = Arc::clone(&metadata);
+        let sw = ShardWriter::initialize(shard_metadata, tsc, psc, vsc, rsc)?;
+        metadata.new_generation_id();
+
+        Ok(sw)
     }
 
     #[measure(actor = "shard", metric = "new")]
-    pub fn new(id: String, path: &Path, metadata: ShardMetadata) -> NodeResult<ShardWriter> {
+    pub fn new(metadata: Arc<ShardMetadata>) -> NodeResult<ShardWriter> {
+        let path = metadata.shard_path();
         let tsc = TextConfig {
             path: path.join(TEXTS_DIR),
         };
@@ -184,7 +191,7 @@ impl ShardWriter {
             path: path.join(PARAGRAPHS_DIR),
         };
 
-        let channel = metadata.channel.unwrap_or_default();
+        let channel = metadata.channel();
 
         let vsc = VectorConfig {
             similarity: Some(metadata.similarity()),
@@ -198,16 +205,15 @@ impl ShardWriter {
         };
 
         std::fs::create_dir(path)?;
-        let metadata_path = path.join(METADATA_FILE);
-        metadata.serialize(&metadata_path)?;
 
-        ShardWriter::initialize(id, path, metadata, tsc, psc, vsc, rsc)
+        let sw = ShardWriter::initialize(Arc::clone(&metadata), tsc, psc, vsc, rsc)?;
+        metadata.serialize_metadata()?;
+        Ok(sw)
     }
 
     #[measure(actor = "shard", metric = "open")]
-    pub fn open(id: String, path: &Path) -> NodeResult<ShardWriter> {
-        let metadata_path = path.join(METADATA_FILE);
-        let metadata = ShardMetadata::open(&metadata_path)?;
+    pub fn open(metadata: Arc<ShardMetadata>) -> NodeResult<ShardWriter> {
+        let path = metadata.shard_path();
         let tsc = TextConfig {
             path: path.join(TEXTS_DIR),
         };
@@ -216,7 +222,7 @@ impl ShardWriter {
             path: path.join(PARAGRAPHS_DIR),
         };
 
-        let channel = metadata.channel.unwrap_or_default();
+        let channel = metadata.channel();
 
         let vsc = VectorConfig {
             similarity: None,
@@ -229,7 +235,7 @@ impl ShardWriter {
             channel,
         };
 
-        ShardWriter::initialize(id, path, metadata, tsc, psc, vsc, rsc)
+        ShardWriter::initialize(metadata, tsc, psc, vsc, rsc)
     }
 
     #[measure(actor = "shard", metric = "set_resource")]
@@ -303,7 +309,7 @@ impl ShardWriter {
         paragraph_result?;
         vector_result?;
         relation_result?;
-        self.new_generation_id(); // VERY NAIVE, SHOULD BE DONE AFTER MERGE AS WELL
+        self.metadata.new_generation_id(); // VERY NAIVE, SHOULD BE DONE AFTER MERGE AS WELL
         Ok(())
     }
 
@@ -364,7 +370,7 @@ impl ShardWriter {
         vector_result?;
         relation_result?;
 
-        self.new_generation_id();
+        self.metadata.new_generation_id();
 
         Ok(())
     }
@@ -428,7 +434,7 @@ impl ShardWriter {
         let mut writer = vector_write(&self.vector_writer);
         writer.add_vectorset(setid, similarity)?;
 
-        self.new_generation_id();
+        self.metadata.new_generation_id();
 
         Ok(())
     }
@@ -438,7 +444,7 @@ impl ShardWriter {
         let mut writer = vector_write(&self.vector_writer);
         writer.remove_vectorset(setid)?;
 
-        self.new_generation_id();
+        self.metadata.new_generation_id();
 
         Ok(())
     }
@@ -459,43 +465,21 @@ impl ShardWriter {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn gc(&self) -> NodeResult<()> {
+    pub fn gc(&self) -> NodeResult<GarbageCollectorStatus> {
         let _lock = self.gc_lock.blocking_lock();
-        vector_write(&self.vector_writer).garbage_collection()
-    }
-
-    #[tracing::instrument(skip_all)]
-    pub fn get_generation_id(&self) -> String {
-        // we can not cache this on the ShardWriter instance unless we get swap our
-        // Arc implementation for a RwLock implementation because we'd need
-        // to write the cache on modification
-
-        let filepath = self.path.join(GENERATION_FILE);
-        // check if file does not exist
-        if !filepath.exists() {
-            return self.new_generation_id();
+        let result = vector_write(&self.vector_writer).garbage_collection();
+        match result {
+            Ok(()) => Ok(GarbageCollectorStatus::GarbageCollected),
+            Err(error) => match error.downcast_ref::<VectorErr>() {
+                Some(VectorErr::WorkDelayed) => Ok(GarbageCollectorStatus::TryLater),
+                _ => Err(error),
+            },
         }
-        std::fs::read_to_string(filepath).unwrap()
     }
 
-    #[tracing::instrument(skip_all)]
-    pub fn new_generation_id(&self) -> String {
-        let generation_id = uuid::Uuid::new_v4().to_string();
-        self.set_generation_id(generation_id.clone());
-        generation_id
-    }
-
-    pub fn set_generation_id(&self, generation_id: String) {
-        let filepath = self.path.join(GENERATION_FILE);
-        std::fs::write(filepath, generation_id).unwrap();
-    }
-
-    pub fn get_kbid(&self) -> String {
-        self.metadata.kbid().unwrap_or_default().to_string()
-    }
-
-    pub fn get_similarity(&self) -> Similarity {
-        self.metadata.similarity().into()
+    pub async fn block_shard(&self) -> BlockingToken {
+        let mutex_guard = self.write_lock.lock().await;
+        BlockingToken(mutex_guard)
     }
 
     pub fn get_shard_segments(&self) -> NodeResult<HashMap<String, Vec<String>>> {
@@ -544,3 +528,21 @@ impl ShardWriter {
         Ok(files)
     }
 }
+
+pub enum GarbageCollectorStatus {
+    GarbageCollected,
+    TryLater,
+}
+
+impl From<GarbageCollectorStatus> for nucliadb_core::protos::garbage_collector_response::Status {
+    fn from(value: GarbageCollectorStatus) -> Self {
+        match value {
+            GarbageCollectorStatus::GarbageCollected => {
+                nucliadb_core::protos::garbage_collector_response::Status::Ok
+            }
+            GarbageCollectorStatus::TryLater => {
+                nucliadb_core::protos::garbage_collector_response::Status::TryLater
+            }
+        }
+    }
+}
