@@ -19,9 +19,11 @@
 
 use std::fs;
 use std::io::Cursor;
+use std::path::PathBuf;
 use std::sync::Arc;
 
 use nucliadb_core::protos::*;
+use nucliadb_core::Channel;
 use nucliadb_node::analytics::blocking::send_analytics_event;
 use nucliadb_node::analytics::payload::AnalyticsEvent;
 use nucliadb_node::lifecycle;
@@ -44,6 +46,7 @@ use crate::RawProtos;
 #[derive(Default)]
 pub struct NodeWriter {
     shards: UnboundedShardWriterCache,
+    shards_path: PathBuf,
 }
 
 impl NodeWriter {
@@ -72,8 +75,10 @@ impl NodeWriter {
                 "Unable to initialize writer: {error}"
             )));
         };
+        let shards_path = settings.shards_path();
         Ok(Self {
             shards: UnboundedShardWriterCache::new(settings),
+            shards_path,
         })
     }
 
@@ -82,7 +87,15 @@ impl NodeWriter {
 
         let request =
             NewShardRequest::decode(&mut Cursor::new(metadata)).expect("Error decoding arguments");
-        let metadata = ShardMetadata::from(request);
+        let shard_id = uuid::Uuid::new_v4().to_string();
+        let similarity = VectorSimilarity::from_i32(request.similarity).unwrap();
+        let metadata = ShardMetadata::new(
+            self.shards_path.join(shard_id.clone()),
+            shard_id,
+            Some(request.kbid),
+            similarity.into(),
+            Some(Channel::from(request.release_channel)),
+        );
         let new_shard = self.shards.create(metadata);
         match new_shard {
             Ok(new_shard) => Ok(PyList::new(
@@ -291,7 +304,7 @@ impl NodeWriter {
         let shard = self.obtain_shard(shard_id.id)?;
         let result = shard.gc();
         match result {
-            Ok(()) => {
+            Ok(_) => {
                 let response = EmptyResponse {};
                 Ok(PyList::new(py, response.encode_to_vec()))
             }
